@@ -57,7 +57,8 @@ def convert_table(infile, outfile=None, subtable=None, keys=None, timecols=None,
     """
     import os
     from numcodecs import Blosc
-    import cngi._helper.table_conversion as tblconv
+    import importlib_metadata
+    import cngi._utils._table_conversion as tblconv
 
     # parse filename to use
     infile = os.path.expanduser(infile)
@@ -80,7 +81,7 @@ def convert_table(infile, outfile=None, subtable=None, keys=None, timecols=None,
     if (keys is None) or (type(keys) is str):
         xds = tblconv.convert_simple_table(infile, outfile,
                                            subtable=subtable,
-                                           rowdim='d0' if keys is None else keys,
+                                           #rowdim='d0' if keys is None else keys,
                                            timecols=[] if timecols is None else timecols,
                                            ignore= [] if ignorecols is None else ignorecols,
                                            compressor=compressor,
@@ -97,5 +98,9 @@ def convert_table(infile, outfile=None, subtable=None, keys=None, timecols=None,
                                              compressor=compressor,
                                              chunk_shape=chunk_shape,
                                              nofile=nofile)
+
+    # write sw version that did this conversion to zarr directory
+    with open(outfile + '/.version', 'w') as fid:
+        fid.write('cngi-protoype ' + importlib_metadata.version('cngi-prototype') + '\n')
     
     return xds
