@@ -15,7 +15,7 @@
 this module will be included in the api
 """
 
-def make_image_with_gcf(vis_dataset, gcf_dataset, img_dataset, grid_parms, norm_parms, sel_parms, storage_parms):
+def make_image_with_gcf(mxds, gcf_dataset, img_dataset, grid_parms, norm_parms, vis_sel_parms, img_sel_parms):
     """
     Creates a cube or continuum dirty image from the user specified visibility, uvw and imaging weight data. A gridding convolution function (gcf_dataset), primary beam image (img_dataset) and a primary beam weight image (img_dataset) must be supplied.
     
@@ -66,23 +66,6 @@ def make_image_with_gcf(vis_dataset, gcf_dataset, img_dataset, grid_parms, norm_
          The primary beam image to use for normalization.
     sel_parms['weight_pb'] : str, default ='WEIGHT_PB'
          The primary beam weight image to use for normalization.
-    storage_parms : dictionary
-    storage_parms['to_disk'] : bool, default = False
-        If true the dask graph is executed and saved to disk in the zarr format.
-    storage_parms['append'] : bool, default = False
-        If storage_parms['to_disk'] is True only the dask graph associated with the function is executed and the resulting data variables are saved to an existing zarr file on disk.
-        Note that graphs on unrelated data to this function will not be executed or saved.
-    storage_parms['outfile'] : str
-        The zarr file to create or append to.
-    storage_parms['chunks_on_disk'] : dict of int, default = {}
-        The chunk size to use when writing to disk. This is ignored if storage_parms['append'] is True. The default will use the chunking of the input dataset.
-    storage_parms['chunks_return'] : dict of int, default = {}
-        The chunk size of the dataset that is returned. The default will use the chunking of the input dataset.
-    storage_parms['graph_name'] : str
-        The time to compute and save the data is stored in the attribute section of the dataset and storage_parms['graph_name'] is used in the label.
-    storage_parms['compressor'] : numcodecs.blosc.Blosc,default=Blosc(cname='zstd', clevel=2, shuffle=0)
-        The compression algorithm to use. Available compression algorithms can be found at https://numcodecs.readthedocs.io/en/stable/blosc.html.
-    
     Returns
     -------
     image_dataset : xarray.core.dataset.Dataset
@@ -102,26 +85,44 @@ def make_image_with_gcf(vis_dataset, gcf_dataset, img_dataset, grid_parms, norm_
     from numcodecs import Blosc
     from itertools import cycle
     
-    from ngcasa._ngcasa_utils._store import _store
-    from ngcasa._ngcasa_utils._check_parms import _check_storage_parms, _check_sel_parms, _check_existence_sel_parms
+    from cngi._utils._check_parms import _check_sel_parms, _check_existence_sel_parms
     from ._imaging_utils._check_imaging_parms import _check_grid_parms, _check_norm_parms
     #from ._imaging_utils._gridding_convolutional_kernels import _create_prolate_spheroidal_kernel, _create_prolate_spheroidal_kernel_1D
     from ._imaging_utils._standard_grid import _graph_standard_grid
     from ._imaging_utils._remove_padding import _remove_padding
     from ._imaging_utils._aperture_grid import _graph_aperture_grid
     from ._imaging_utils._normalize import _normalize
+    from cngi.image import make_empty_sky_image
     
+    
+    #Deep copy so that inputs are not modified
+    _mxds = mxds.copy(deep=True)
+    _img_dataset = img_dataset.copy(deep=True)
+    _vis_sel_parms = copy.deepcopy(vis_sel_parms)
+    _img_sel_parms = copy.deepcopy(img_sel_parms)
     _grid_parms = copy.deepcopy(grid_parms)
-    _storage_parms = copy.deepcopy(storage_parms)
-    _sel_parms = copy.deepcopy(sel_parms)
     _norm_parms = copy.deepcopy(norm_parms)
     
-    assert(_check_sel_parms(_sel_parms,{'uvw':'UVW','data':'DATA','imaging_weight':'IMAGING_WEIGHT','sum_weight':'SUM_WEIGHT','image':'IMAGE','pb':'PB','weight_pb':'WEIGHT_PB'})), "######### ERROR: sel_parms checking failed"
-    assert(_check_existence_sel_parms(vis_dataset,{'uvw':_sel_parms['uvw'],'data':_sel_parms['data'],'imaging_weight':_sel_parms['imaging_weight']})), "######### ERROR: sel_parms checking failed"
-    assert(_check_existence_sel_parms(img_dataset,{'pb':_sel_parms['pb'],'weight_pb':_sel_parms['weight_pb']})), "######### ERROR: sel_parms checking failed"
+    ##############Parameter Checking and Set Defaults##############
+    assert('xds' in _vis_sel_parms), "######### ERROR: xds must be specified in sel_parms" #Can't have a default since xds names are not fixed.
+    _vis_dataset = _mxds.attrs[_vis_sel_parms['xds']]
+    
     assert(_check_grid_parms(_grid_parms)), "######### ERROR: grid_parms checking failed"
     assert(_check_norm_parms(_norm_parms)), "######### ERROR: norm_parms checking failed"
-    assert(_check_storage_parms(_storage_parms,'dirty_image.img.zarr','make_image')), "######### ERROR: storage_parms checking failed"
+    
+    #Check vis data_group
+    _check_sel_parms(_vis_dataset,_vis_sel_parms)
+    
+    #Check img data_group
+ 
+    _check_sel_parms(_img_dataset,_img_sel_parms,new_or_modified_data_variables={'sum_weight':'SUM_WEIGHT','image':'IMAGE'},required_data_variables={'pb':'PB','weight_pb':'WEIGHT_PB'},append_to_in_id=False)
+    #'pb':'PB','weight_pb':'WEIGHT_PB',
+    #print('did this work',_img_sel_parms)
+    
+    
+#    assert(_check_sel_parms(_sel_parms,{'uvw':'UVW','data':'DATA','imaging_weight':'IMAGING_WEIGHT','sum_weight':'SUM_WEIGHT','image':'IMAGE','pb':'PB','weight_pb':'WEIGHT_PB'})), "######### ERROR: sel_parms checking failed"
+#    assert(_check_existence_sel_parms(vis_dataset,{'uvw':_sel_parms['uvw'],'data':_sel_parms['data'],'imaging_weight':_sel_parms['imaging_weight']})), "######### ERROR: sel_parms checking failed"
+#    assert(_check_existence_sel_parms(img_dataset,{'pb':_sel_parms['pb'],'weight_pb':_sel_parms['weight_pb']})), "######### ERROR: sel_parms checking failed"
     
     # Creating gridding kernel
     #cgk, correcting_cgk_image = _create_prolate_spheroidal_kernel(_grid_parms['oversampling'], _grid_parms['support'], _grid_parms['imsize_padded'])
@@ -136,40 +137,63 @@ def make_image_with_gcf(vis_dataset, gcf_dataset, img_dataset, grid_parms, norm_
     _grid_parms['do_psf'] = False
     _grid_parms['oversampling'] = np.array(gcf_dataset.oversampling)
 
-    grids_and_sum_weights = _graph_aperture_grid(vis_dataset,gcf_dataset,_grid_parms,_sel_parms)
+    grids_and_sum_weights = _graph_aperture_grid(_vis_dataset,gcf_dataset,_grid_parms,_vis_sel_parms)
     uncorrected_dirty_image = dafft.fftshift(dafft.ifft2(dafft.ifftshift(grids_and_sum_weights[0], axes=(0, 1)), axes=(0, 1)), axes=(0, 1))
         
     #Remove Padding
     print('grid sizes',_grid_parms['image_size_padded'][0], _grid_parms['image_size_padded'][1])
     uncorrected_dirty_image = _remove_padding(uncorrected_dirty_image,_grid_parms['image_size']).real * (_grid_parms['image_size_padded'][0] * _grid_parms['image_size_padded'][1])
-    normalized_image = _normalize(uncorrected_dirty_image, grids_and_sum_weights[1], img_dataset, gcf_dataset, 'forward', _norm_parms, sel_parms)
+    
+    #print(_img_sel_parms)
+    normalized_image = _normalize(uncorrected_dirty_image, grids_and_sum_weights[1], img_dataset, gcf_dataset, 'forward', _norm_parms, _img_sel_parms)
     
     if _grid_parms['chan_mode'] == 'continuum':
-        freq_coords = [da.mean(vis_dataset.coords['chan'].values)]
-        chan_width = da.from_array([da.mean(vis_dataset['chan_width'].data)],chunks=(1,))
+        freq_coords = [da.mean(_vis_dataset.coords['chan'].values)]
+        chan_width = da.from_array([da.mean(_vis_dataset['chan_width'].data)],chunks=(1,))
         imag_chan_chunk_size = 1
     elif _grid_parms['chan_mode'] == 'cube':
-        freq_coords = vis_dataset.coords['chan'].values
-        chan_width = vis_dataset['chan_width'].data
-        imag_chan_chunk_size = vis_dataset.DATA.chunks[2][0]
+        freq_coords = _vis_dataset.coords['chan'].values
+        chan_width = _vis_dataset['chan_width'].data
+        imag_chan_chunk_size = _vis_dataset.DATA.chunks[2][0]
     
     ###Create Image Dataset
-    chunks = vis_dataset.DATA.chunks
+    chunks = _vis_dataset.DATA.chunks
     n_imag_pol = chunks[3][0]
     
-    coords = {'d0': np.arange(_grid_parms['image_size'][0]), 'd1': np.arange(_grid_parms['image_size'][1]),
-              'chan': freq_coords, 'pol': np.arange(n_imag_pol), 'chan_width' : ('chan',chan_width)}
-    img_dataset = img_dataset.assign_coords(coords)
-    img_dataset[_sel_parms['sum_weight']] = xr.DataArray(grids_and_sum_weights[1], dims=['chan','pol'])
-    img_dataset[_sel_parms['image']] = xr.DataArray(normalized_image, dims=['d0', 'd1', 'chan', 'pol'])
+    #coords = {'d0': np.arange(_grid_parms['image_size'][0]), 'd1': np.arange(_grid_parms['image_size'][1]),
+    #          'chan': freq_coords, 'pol': np.arange(n_imag_pol), 'chan_width' : ('chan',chan_width)}
+    #img_dataset = img_dataset.assign_coords(coords)
+    #img_dataset[_sel_parms['sum_weight']] = xr.DataArray(grids_and_sum_weights[1], dims=['chan','pol'])
+    #img_dataset[_sel_parms['image']] = xr.DataArray(normalized_image, dims=['d0', 'd1', 'chan', 'pol'])
+    
+    
+    phase_center = _grid_parms['phase_center']
+    image_size = _grid_parms['image_size']
+    cell_size = _grid_parms['cell_size']
+    phase_center = _grid_parms['phase_center']
+
+    pol_coords = _vis_dataset.pol.data
+    time_coords = [_vis_dataset.time.mean().data]
+    
+    _img_dataset = make_empty_sky_image(_img_dataset,phase_center,image_size,cell_size,freq_coords,chan_width,pol_coords,time_coords)
+    
+    _img_dataset[_img_sel_parms['data_group_out']['sum_weight']] = xr.DataArray(grids_and_sum_weights[1][None,:,:], dims=['time','chan','pol'])
+    _img_dataset[_img_sel_parms['data_group_out']['image']] = xr.DataArray(normalized_image[:,:,None,:,:], dims=['l', 'm', 'time', 'chan', 'pol'])
+    _img_dataset.attrs['data_groups'][0] = {**_img_dataset.attrs['data_groups'][0],**{_img_sel_parms['data_group_out']['id']:_img_sel_parms['data_group_out']}}
+    
+    
+    #list_xarray_data_variables = [img_dataset[_sel_parms['image']],img_dataset[_sel_parms['sum_weight']]]
+    #return _store(img_dataset,list_xarray_data_variables,_storage_parms)
+    
+    print('#########################  Created graph for make_mosaic_with_gcf #########################')
+    return _img_dataset
+    
     
-    list_xarray_data_variables = [img_dataset[_sel_parms['image']],img_dataset[_sel_parms['sum_weight']]]
-    return _store(img_dataset,list_xarray_data_variables,_storage_parms)
     
     
     '''
     ###Create Dataset
-    chunks = vis_dataset.DATA.chunks
+    chunks = _vis_dataset.DATA.chunks
     n_imag_pol = chunks[3][0]
     image_dict = {}
     coords = {'d0': np.arange(_grid_parms['image_size'][0]), 'd1': np.arange(_grid_parms['image_size'][1]),
